# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

version <- function() {
    .Call('_V8_version', PACKAGE = 'V8')
}

context_eval <- function(src, ctx) {
    .Call('_V8_context_eval', PACKAGE = 'V8', src, ctx)
}

context_validate <- function(src, ctx) {
    .Call('_V8_context_validate', PACKAGE = 'V8', src, ctx)
}

context_null <- function(ctx) {
    .Call('_V8_context_null', PACKAGE = 'V8', ctx)
}

make_context <- function(set_console) {
    .Call('_V8_make_context', PACKAGE = 'V8', set_console)
}

context_enable_typed_arrays <- function(ctx) {
    .Call('_V8_context_enable_typed_arrays', PACKAGE = 'V8', ctx)
}

